<?php

namespace Cms\Chords;

/**
 * SongController
 *
 * @package RynelleCMS
 * @author  Rynelle Coronacion <dev.rynelle@gmail.com>
 * @version v1
 */

use Cms\CmsBaseController;

use Input;
use Libraries\Validator\Rules;
use Libraries\Chords\Scraper;

use Repositories\Chords\ArtistRepository;
use Repositories\Chords\SongRepository;
use Repositories\Chords\TypeRepository;

class SongController extends CmsBaseController {


	/**
	 * Set Global View Path for Settings
	 */
	protected $view_path = 'chords.songs';

	/**
	 * Repositories\Chords\ArtistRepository;
	 */
	protected $_artist;

	/**
	 * Repositories\Chords\SongRepository;
	 */
	protected $_song;

	/**
	 * Repositories\Chords\TypeRepository;
	 */
	protected $_type;

	/**
	 * Libraries\Chords\Scraper;
	 */
	protected $_scraper;

	/**
	 * Controller Initializer
	 *
	 * @return void
	 */
	public function __construct( 
								  SongRepository $_song,
								  ArtistRepository $_artist,
								  TypeRepository $_type,
								  Scraper $_scraper
							   )
	{
		$this->_song    = $_song;
		$this->_artist  = $_artist;
		$this->_type    = $_type;
		$this->_scraper = $_scraper;
	}

	/**
	 * Display Index Page
	 *
	 * @return void
	 */
	public function getIndex()
	{

		$types = $this->_type->collections();

		$data  = [
					'types' => json_encode($types)
				 ];
		return $this->view('index', $data);
	}

	/**
	 * Get Collections
	 *
	 * @return void
	 */
	public function postArtists()
	{
		$artists = $this->_artist->collections([], true);

		$artists[] = [
						'_id'   => 'others',
			            'name'  => 'Others',
			            'group' => 'O'
					 ];
		return $this->response($artists);
	}

	/**
	 * Get Collections
	 *
	 * @return void
	 */
	public function postCollection()
	{

		$input   = Input::all();
		$songs = $this->_song->collections($input);
        $data  	 = [];
        foreach ($songs as $song)
        {
            $data[] =  [
                            'id'     	 => encrypt($song->id),
                            'title'      => $song->title,
                            'slug'  	 => $song->slug,
                            'version'  	 => $song->version,
                            'capo'		 => $song->capo,
                            'type'		 => $song->type->id,
                            'difficulty' => $song->difficulty,
                            'tabs' 		 => $song->tabs,
                            'chords' 	 => $song->chords,
                            'audio' 	 => $song->audio,
                            'video' 	 => $song->video,
                            'type'   	 => [
                            					'_id'  => $song->type->id,
                            					'name' => $song->type->name,
                            					'slug' => $song->type->slug
                            				],
                            'artist' 	 => [
                            					'_id'  => $song->artist->id,
                            					'name' => $song->artist->name,
                            					'slug' => $song->artist->slug
                            				]
                        ];
        }

        $songs->setItems( $data );

		return $this->response($songs);

	}


	/**
	 * Get Collections
	 *
	 * @return void
	 */
	public function postMultiScrap()
	{
		$input = Input::all();
		if ( $input ) {

			$search = trim($input['search']);
			$page   = intval($input['page']);

			$songs = $this->_scraper->search($search, $page);

			return $this->response($songs);
		}

	}


	/**
	 * Form Update
	 *
	 * @return void
	 */
	public function postSubmit($input = null)
	{
		$input = $input ?: Input::all();
		$reserve_words = reserve_words();


		if ( $input ) {

			$id = ( @$input['_id'] ) ? decrypt($input['_id']) : 0;
			
			$input['artist_id'] = ( $input['artist'] == 'others' ) ? 'others' : intval(@$input['artist']);
			$input['type_id'] = intval(@$input['type']) ?: 1;

			unset($input['type']);
			unset($input['artist']);
			$message = 'New Song Successfully Created.';

            $rules = [
                        'title' 	 => 'required',
                        'slug' 		 => 'required|unique:songs,slug,NULL,NULL,deleted_at,NULL',
                        'chords' 	 => 'required',
                        'version'  	 => 'required',
                        'difficulty' => 'required',
                     ];
            if ( $id ) {
            	$rules['slug'] = 'required';
            	$message 	   = 'Song Successfully Updated.';
            }

			$validate = Rules::validate($input, $rules);

			if ( $input['artist_id'] === 0 || ( $input['artist_id'] === 'others' && strlen(trim(@$input['artist_other'])) == 0 ) )
				$validate['errors']['artist'] = 'The artist field is required.';

			if ( $input['video'] ) {
				if ( strpos($input['video'], 'youtube.com') === false ) {
					$validate['errors']['video'] = 'The video field is invalid.';
				}
			}

			if ( $input['slug'] ) {
				if ( in_array($input['slug'], $reserve_words) )
					$validate['errors']['slug'] = 'The slug you entered is reserved word.';
			}

			if ( $validate )
				return $this->response($validate);

			if ( $input['artist_id'] === 'others' ) {
				$artist_other = trim(@$input['artist_other']); 
				$artistwhere  = [ 'slug' => str_slug($artist_other) ];
				$artistdata   = [ 'name' => $artist_other ];
				$artist 	  = $this->_artist->save($artistdata, $artistwhere);
				$input['artist_id'] = $artist->id;
			}

			$song = $this->_song->save($input, $id);

			return $this->response($message);

		}
	}

    /**
    * Delete Song
    * Set deleted_at as deleted
    *
    * @return void
    * @access private
    **/
    public function postDeleteSong()
    {

		$input = Input::all();

		if ( $input ) {
			$id  = decrypt($input['_id']);
			$this->_song->delete($id);
		}

		return $this->response(true);

    }

    /**
    * Scrap Url
    *
    * @return void
    * @access private
    **/
    public function postScrapUrl()
    {

		$input    = Input::all();
		$response = false;

		if ( $input ) {
			$validate = Rules::validate($input, [ 'url'=> 'required|url' ]);
			if ( $validate )
				return $this->response($validate);

			$url      = $input['url'];
			$autosave = @$input['autosave'] == 'save';
			$scraper = $this->_scraper->init($url);
			
			if ( $scraper->validate() ) {
				$response = [];

				$scrap = [
								'title' 	 => $scraper->title(),
								'type' 	 	 => $scraper->type(),
								'version' 	 => $scraper->version(),
								'difficulty' => $scraper->difficulty(),
								'chords' 	 => $scraper->chords(),
							];

				$slug   = str_slug($scrap['title']);
				$songs  = $this->_song->get($slug, 'slug', 'LIKE', false);
				$chords = [];
				$slug_total = 0;
				if ( $songs ) {
					foreach ($songs as $song) {
						$chords[] = [
										'name'    => $song->title,
										'link'    => $song->slug,
										'version' => $song->version,
									];
						$slug_total++;
					}
				}
				if ( $slug_total > 0 ) {
					$slug_version = intval($slug_total) + 1;
					$slug .= '-';
					$slug .= $slug_version;
					$scrap['version'] = $slug_version;
				}
				$scrap['slug'] = $slug;

				$res_author = $scraper->author();
				$author 	   = str_slug($res_author);

				$artist_id    = 'others';
				$artist_name  = $res_author;
				$artist_group = '';
				$artist = $this->_artist->getartist($author);
				if ( $artist ) {
					$artist_id    = $artist->id;
					$artist_name  = $artist->name;
					$artist_group = ''; // strtoupper(substr($artist->name, 0, 1));
				}
				$scrap['artist'] = [
										'_id'   => $artist_id,
							            'name'  => $artist_name,
							            'group' => $artist_group,
									  ];

				$response['form']   = $scrap;
				$response['chords'] = $chords;

				if ( $autosave === true ) {

					$title   = $scrap['title'];
					$version = $scrap['version'];
					if ( $version > 1 )
						$title .= ' v' . $version;

					$slug   	= str_slug($title);
					$submitdata = [
				                    'title'        => $title,
				                    'slug'         => $slug,
				                    'artist'       => $artist_id,
				                    'artist_other' => $res_author,
				                    'chords'       => $scrap['chords'],
				                    'capo'         => 0,
				                    'version'      => $scrap['version'],
				                    'type'         => $scrap['type'],
				                    'difficulty'   => $scrap['difficulty'],
				                    'audio'        => '',
				                    'video'        => '',
				                  ];

			        if ( $artist_id != 'others' ) {
			        	$songwhere = [
			        					 'slug'	     => $slug,
			        					 'artist_id' => $artist_id
			        				 ];
						$song = $this->_song->get($songwhere);
						if ( $song )
							$submitdata['_id'] = encrypt($song->id);
			        }

			        $this->postSubmit($submitdata);

					$response['_id']  = @$input['_id'];
					$response['page'] = @$input['page'];

				}
				
			}
			else {
				$response['errors']['url'] = 'The page you are looking for doesn\'t exist.';
			}

		}

		return $this->response($response);

    }

}