<?php

namespace Repositories\Users;

/**
 * AuthRepository
 *
 * @package RynelleCMS
 * @author  Rynelle Coronacion <dev.rynelle@gmail.com>
 * @version v1
 */

use SysResponse;
use Request;
use Sentry;
use Session;
use Input;
use Config;

use Repositories\Base\Repository;

class AuthRepository extends Repository
{

    /**
    * __construct()
    * Initialize our Class Here for Dependecy Injection
    *
    * @return void
    * @access  public
    **/
    public function __construct()
    {
        $this->credentials = Input::all();
        $this->userLogged  = null;
        $this->userGroup   = null;
        $this->rememberme  = false;
    }

    /**
    * authenticate()
    *
    *
    * @return void
    * @access  public
    **/
    public function authenticate()
    {
        try
        {

            $this->authAttribute();

            unset($this->credentials['_token']);

            // Authenticate the user
            $this->userLogged = Sentry::authenticate($this->credentials, $this->rememberme);
            $this->userGroup  = @$this->userLogged->getGroups()->first()->name;

            $data = [
                        'success' => true,
                        'name'    => $this->userLogged->first_name,
                    ];
            $this->setUserLoggedSession();
        }
        catch (\Cartalyst\Sentry\Users\LoginRequiredException $e)
        {
            $data = [
                        'success' => false,
                        'error'   => 'Login field is required.'
                    ];
        }
        catch (\Cartalyst\Sentry\Users\PasswordRequiredException $e)
        {
            $data = [
                        'success' => false,
                        'error'   => 'Password field is required.'
                    ];
        }
        catch (\Cartalyst\Sentry\Users\WrongPasswordException $e)
        {
            $data = [
                        'success' => false,
                        'error'   => 'Wrong password, try again.'
                    ];
        }
        catch (\Cartalyst\Sentry\Users\UserNotFoundException $e)
        {
            $data = [
                        'success' => false,
                        'error'   => 'User was not found.'
                    ];
        }
        catch (\Cartalyst\Sentry\Users\UserNotActivatedException $e)
        {
            $data = [
                        'success' => false,
                        'error'   => 'User is not activated.'
                    ];
        }

        // The following is only required if the throttling is enabled
        catch (\Cartalyst\Sentry\Throttling\UserSuspendedException $e)
        {
            $data = [
                        'success' => false,
                        'error'   => 'User is suspended.'
                    ];
        }
        catch (\Cartalyst\Sentry\Throttling\UserBannedException $e)
        {
            $data = [
                        'success' => false,
                        'error'   => 'User is banned.'
                    ];
        }

        return $data;

    }

    /**
    * setUserLoggedSession()
    * Set to Session the current user that log-in., with
    *
    * @return void
    * @access  public
    **/
    private function setUserLoggedSession()
    {
        $session = [
            'logged'        => true,
            'locked'        => false,
            'hashid'        => encrypt($this->userLogged->id),
            'username'      => $this->userLogged->username,
            'email'         => $this->userLogged->email,
            'last_login'    => $this->userLogged->last_login,
            'firstname'     => $this->userLogged->first_name,
            'lastname'      => $this->userLogged->last_name,
            'fullname'      => $this->userLogged->first_name . ' ' . $this->userLogged->last_name,
            'avatar'        => img_placeholder('160x160', substr(strtolower($this->userLogged->first_name), 0, 1), true),
            'group'         => ucfirst($this->userGroup),
        ];

        return Session::put(SESSION_CMS_USERINFO, $session);
    }

    /**
    * authAttribute()
    *
    * @return void
    * @access  public
    **/
    private function authAttribute()
    {
        $identity = $this->credentials['identity'];
        $loginAttribute = filter_var($identity, FILTER_VALIDATE_EMAIL) && preg_match('/@.+\./', $identity) ? 'email' : 'username';

        $this->credentials[$loginAttribute] = $identity;

        if( @$this->credentials['rememberme'] == 'true')
        {
            $this->rememberme = true;
        }

        unset($this->credentials['identity']);
        unset($this->credentials['rememberme']);

        $path = Request::path();
        unset($this->credentials["/{$path}"]);

        // Get Only Login Attribute and password for Sentry authenticating.
        $this->input = array_only($this->credentials, [$loginAttribute, 'password']);
        Config::set('cartalyst/sentry::users.login_attribute', $loginAttribute);
        return;

    }

    /**
    * resetSession()
    * Reset session when user info is updated
    *
    * @return void
    * @access  public
    **/
    public function resetSession($userId)
    {
        $this->userLogged = Sentry::findUserById($userId);
        $this->setUserLoggedSession();
    }

    /**
    *
    * logout()
    *
    * @return void
    * @access  public
    **/
    public function logout()
    {

        Session::forget(SESSION_CMS_USERINFO);
        Sentry::logout();

        return true;
    }

}