<?php

namespace Repositories\Base;

/**
 * The Abstract Repository provides default implementations of the methods defined
 * in the base repository interface. These simply delegate static function calls
 * to the right eloquent model based on the $modelclass.
 *
 * @author  Rynelle Coronacion <dev.rynelle@gmail.com>
 * @version v1
**/

use Illuminate\Container\Container;
use Illuminate\Database\Eloquent\ModelNotFoundException;

use Illuminate\Database\QueryException;

abstract class Repository implements RepositoryInterface {

    /**
     * @var string
     */
    public $perpage = 10;

    /**
     * @var string
     */
    protected $modelclass;

    /**
     * @var \Illuminate\Database\Eloquent\Model
     */
    protected $model;

    public function __construct()
    {
    	$app = new Container;
        $this->model = $app->make($this->modelclass);
    }

    /**
     * Make a new instance of the entity to query on
     *
     * @param array $attributes = array()
     * @return static
     */
 	public function getInstance(array $attributes = array())
 	{
 		return $this->model->newInstance( $attributes );
 	}

    /**
     * Eager Load, Model Relationship
     *
     * @param array $with = array()
     * @return \Illuminate\Database\Eloquent\Builder|static
     */
	public function with(array $with = array())
	{
	  return $this->model->with($with);
	}

    /**
     * Call Create Instance and prepare for inserting in our Model.
     *
     * @param array $attributes
     * @return static
     */
	public function create(array $attributes)
	{
		return $this->model->create($attributes);
	}

    /**
     * Make a new instance of the entity to query on
     *
     * @param array $columns
     * @return \Illuminate\Database\Eloquent\Collection|static[]
     */
	public function all($columns = array('*'))
	{
		return $this->model->all($columns);
	}

    /**
     * Find specific data by $id.
     *
     * @param $id , $columns = array('*')
     * @return \Illuminate\Support\Collection|null|static
     */
	public function find($id, $columns = array('*'))
	{
		return $this->model->find($id, $columns);
	}

    /**
     * Find a model by its primary key or throw an exception.
     *
     * @param  mixed $id
     * @param  array $columns
     *
     * @return \Illuminate\Support\Collection|null|static
     * @throws ModelNotFoundException
     */
	public function findOrFail($id, $columns = array('*'))
	{
		if ( ! is_null($model = $this->model->find($id, $columns))) return $model;

		throw (new ModelNotFoundException)->setModel(get_called_class());
	}


    /**
     * Make an update for specific model by its attributes
     *
     * @param array $attributes = array(
     * @return bool|int
     */
	public function update(array $attributes = array())
	{
		return $this->model->update($attributes);
	}

	/**
	 * Create or update a record matching the attributes, and fill it with values.
	 *
	 * @param  array  $attributes
	 * @param  array  $values
	 * @return static
	 */
	public function updateOrCreate(array $attributes, array $values = array())
	{
		return $this->model->updateOrCreate($attributes,$values);
	}


    /**
     * Paginate Model paginateItems..
     *
     * @param array|int $paginateItems
     * @return \Illuminate\Pagination\Paginator
     */
	public function paginate($paginateItems = 25)
	{
		return $this->model->paginate($paginateItems);
	}

    /**
     * Destroy Current Model by ids..
     *
     * @param array $ids
     * @return int
     */
	public function destroy($ids)
	{
		return $this->model->destroy($ids);
	}

    /*
     * Save Data

     * @param array  $data
     * @param string $value
     * @param string $field
     * @return int
     */
    public function save($data, $value = null, $field = 'id')
    {
        unset($data['_id']);
        unset($data['_token']);

        if ( $value )
        {
            if ( !is_array($value) )
            {
                $value = [ $field => $value ];
            }
            return $this->model->updateOrCreate($value, $data);
        }
        return $this->model->create($data);
    }

    /**
     * Delete
     *
     * @param array $ids
     * @return int
     */
    public function delete($value, $field = 'id')
    {
        return $this->model->where($field, $value)->delete();
    }

    /**
     * Count
     *
     * @param array $ids
     * @return int
     */
    public function get($data, $field = 'id', $action = '=', $first = true)
    {
        if ( is_array($data) ) {
            $query = $this->model;
            foreach ($data as $field => $value) {
                $action_field = explode('|', $field);
                $where_field  = $action_field[0];
                $where_action = @$action_field[1] ?: '=';
                $query = $query->where($where_field, $where_action, $value);
            }
        }
        else {
            if ( strtolower($action) == 'like' )
                $data = "%$data%";
            $query = $this->model->where($field, $action, $data);
        }

        if ( $first === false )
            return $query->get();

        return $query->first();
    }

    /**
     * Get Latest
     *
     * @param array $ids
     * @return int
     */
    public function latest($with = null, $where = null, $limit = 25, $is_random = false)
    {
        $query = $this->model;

        if ( $with !== null )
            $query = $query->with($with);

        if ( is_array($where) ) {
            foreach ($where as $key => $value) {

                $field_action = explode('|', $key);

                $field  = $field_action[0];
                $action = @$field_action[1] ?: '=';

                $query = $query->where($field, $action, $value);
            }
        }

        $query = ( $is_random === true ) ? $query->orderByRaw("RAND()") : $query->orderBy('created_at', 'DESC');

        return $query->limit($limit)->get();
    }

    /**
     * Get Latest
     *
     * @param array $ids
     * @return int
     */
    public function random($with = null, $where = null, $limit = 25)
    {
        return $this->latest($with, $where, $limit, true);
    }

    /**
     * Count
     *
     * @param array $ids
     * @return int
     */
    public function total($value = null, $field = 'id', $action = '=')
    {
        if ( strtolower($action) == 'like' )
            $value = "%$value%";

        $query = $this->model;

        if ( $value !== null )
            $query = $query->where($field, $action, $value);

        return $query->count();
    }

}